function [coord, tri] = gen_coord_random_2d(n, T)
%function [coord, tri] = gen_coord_random_2d(n, T)
%
% This function generates a set of n 2D random points in a fixed
% domain T. In addition, it generates a set of points forming an
% artificial boundary that encloses the n points. A mesh of the
% points is also returned.
%
% INPUTS
%   n:     Number of points, excluding the artificial boundary.
%   T:     Physical domain. E.g., [50;50].
%
% OUTPUTS
%   coord: Coordinates of the points, including boundary.
%   tri:   Triangles of the mesh. Each row represents one triangle.
%

%Author: J. Chen
%$Date: 2011/11/23 11:45:24$

sn = ceil(sqrt(n));
delta = 1/sn;

x = [rand(n,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta; ...
     (1+delta); ...
     -delta; ...
     (1+delta)];

y = [rand(n,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     -delta; ...
     -delta; ...
     (1+delta); ...
     (1+delta)];

if length(T) == 1
  T = repmat(T,1,2);
end

x = x*T(1);
y = y*T(2);

coord = [x y];
if nargout > 1
  tri = delaunay(x,y);
end
