function [coord, tetra] = gen_coord_random_3d(n, T)
%function [coord, tetra] = gen_coord_random_3d(n, T)
%
% This function generates a set of n 3D random points in a fixed
% domain T. In addition, it generates a set of points forming an
% artificial boundary that encloses the n points. A mesh of the
% points is also returned.
%
% INPUTS
%   n:     Number of points, excluding the artificial boundary.
%   T:     Physical domain. E.g., [50;50;50].
%
% OUTPUTS
%   coord: Coordinates of the points, including boundary.
%   tetra: Tetrahedrons of the mesh. Each row represents one tetrahedron.
%

%Author: J. Chen
%$Date: 2011/11/23 11:45:41$

sn = ceil(n.^(1/3));
delta = 1/sn;

x = [rand(n,1); ...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     -delta*ones(sn^2,1); ...
     (1+delta)*ones(sn^2,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta; ...
     (1+delta); ...
     (1+delta); ...
     -delta; ...
     -delta; ...
     -delta; ...
     (1+delta); ...
     (1+delta)];

y = [rand(n,1); ...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     -delta*ones(sn^2,1); ...
     (1+delta)*ones(sn^2,1);...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     (1+delta)*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     (1+delta)*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     (1+delta); ...
     (1+delta); ...
     -delta; ...
     -delta; ...
     -delta; ...
     (1+delta); ...
     (1+delta); ...
     -delta];

z = [rand(n,1); ...
     -delta*ones(sn^2,1); ...
     (1+delta)*ones(sn^2,1); ...
     rand(sn^2,1); ...
     rand(sn^2,1);...
     rand(sn^2,1); ...
     rand(sn^2,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     -delta*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     (1+delta)*ones(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     rand(sn,1); ...
     -delta; ...
     -delta; ...
     -delta; ...
     -delta; ...
     (1+delta); ...
     (1+delta); ...
     (1+delta); ...
     (1+delta)];

if length(T) == 1
  T = repmat(T,1,3);
end

x = x*T(1);
y = y*T(2);
z = z*T(3);

coord = [x y z];
if nargout > 1
  %tetra = delaunay3(x,y,z);
  tetra = DelaunayTri(x,y,z);
  tetra = tetra(:,:);
end
