function [T] = toe2mat(t)
%function [T] = toe2mat(t)
%
% This function returns the symmetric multilevel Toeplitz matrix given
% its first column.
%
% INPUTS
%   t: First column of T, reshaped to the size of each level.
%
% OUTPUTS
%   T: The Toeplitz matrix.
%

%Author: J. Chen
%$Date: 2011/11/23 11:52:19$

dim = size(t);
d = length(dim);

idx = gen_idx(dim(1));
for i = 2:d
  n = size(idx,1);
  idx = kron( ones(dim(i)), idx ) + ...
        kron( gen_idx(dim(i))*n, ones(n) );
end
idx = idx + 1;

t = t(:);
T = t(idx);


function [idx] = gen_idx(n)

idx = toeplitz(0:n-1);
