function [I, V] = score_func_mle_covar(K, dK, N)
%function [I, V] = score_func_mle_covar(K, dK, N)
%
% This function returns the Fisher information matrix I, and the
% covariance matrix in the convergence result of the MLE estimate by
% using the SAA score equation approach. This function computes the
% matrices exactly, thus is only useful for small K. To compute the
% matrices approximately, use the function
% SCORE_FUNC_MLE_COVAR_APPROX.
%
% INPUTS
%   K, dK: Covariance matrix and derivatives. CANNOT be function handles.
%   N:     Appear in the SAA of the score function.
%
% OUTPUTS
%   I:     Fisher information matrix.
%   V:     Covariance matrix in convergence result.
%

%Author: J. Chen
%$Date: 2011/12/19 17:32:11$

d = length(dK);

for i = 1:d
  W{i} = K\dK{i};
end

I = zeros(d);
J = zeros(d);
for i = 1:d
  for j = 1:d
    I(i,j) = sum(diag(W{i}*W{j}))/2;
    J(i,j) = I(i,j)/2 + sum(diag(W{i}*W{j}'))/4 - diag(W{i})'*diag(W{j})/2;
  end
end
I = (I+I')/2;
J = (J+J')/2;

V = (eye(d) + (I\J)/N) * inv(I);
V = (V+V')/2;