function [M] = stiff2d(coord, tri, boundary)
%function [M] = stiff2d(coord, tri, boundary)
%
% This function computes the stiffness matrix M for a 2D mesh.
%
% INPUTS
%   coord:    Coordinates of the points.
%   tri:      Tris of the mesh.
%   boundary: Indicies of the boundary points of the mesh.
%
% OUTPUTS
%   M:        Stiffness matrix, sparse.
%

%Author: J. Chen
%$Date: 2011/11/23 11:54:45$

n = size(coord,1);
A = zeros(size(tri,1)*9, 3);

cnt = 1;
for i = 1:size(tri,1)
  id = tri(i,:);
  B = [coord(id(1),:)' coord(id(2),:)' coord(id(3),:)'; ...
       ones(1,3)];
  
  C = zeros(2,3);
  for j = 1:2
    for k = 1:3
      C(j,k) = (-1)^(j+k)*det(B([1:j-1 j+1:end],[1:k-1 k+1:end]));
    end
  end
  
  detB2 = abs(det(B))*2;
  
  for j = 1:3
    for k = 1:3
      A(cnt,:) = [id(j) id(k) C(:,j)'*C(:,k)/detB2];
      cnt = cnt + 1;
    end
  end
end

M = spconvert([A; n n 0]);
M(:,boundary) = [];
M = M';
M(:,boundary) = [];
M = M';
